@@ -54,6 +54,10 @@ export class ChatHistoryManager {
     // );
   }
 
+  public getCurrentThreadId(): string | null {
+    return this.currentThreadId;
+  }
+
   public updateNotebookId(oldId: string, newId: string): void {
     this.currentNotebookId = newId;
     const threads = this.notebookChats.get(oldId) || [];
