@@ -2,6 +2,7 @@ import { Cell } from '@jupyterlab/cells';
 import { Notebook, NotebookActions } from '@jupyterlab/notebook';
 import { NotebookTools } from './NotebookTools';
 import { AppStateService } from '../AppState';
+import { ActionType } from '../Chat/ActionHistory';
 
 /**
  * Class providing cell manipulation tools for notebooks
@@ -312,8 +313,10 @@ export class NotebookCellTools {
       metadata.custom = {};
     }
 
-    if (metadata && metadata.cell_tracker && metadata.cell_tracker.trackingId)
-      metadata.cell_tracker.trackingId = 'planning_cell';
+    if (!metadata.cell_tracker) {
+      metadata.cell_tracker = {};
+    }
+    metadata.cell_tracker.trackingId = 'planning_cell';
 
     metadata.custom.sage_cell_type = 'plan';
     firstCell.model.sharedModel.setMetadata(metadata);
@@ -581,43 +584,48 @@ export class NotebookCellTools {
   }
 
   /**
-   * Modifies the plan of the notebook by updating the first cell and optionally adding current/next step cells.
+   * Generates and updates the notebook plan based on immediate action.
    *
    * @param options Configuration options
-   * @param options.updated_plan_string Updated plan string in markdown format
-   * @param options.current_step_string Current step string (optional)
-   * @param options.next_step_string Next step string (optional)
+   * @param options.immediate_action What the LLM is about to do next
    * @param options.notebook_path Path to the notebook file (optional)
-   * @returns True if the plan was updated successfully, False otherwise
+   * @returns The generated plan string
    */
-  edit_plan(options: {
-    updated_plan_string: string;
-    current_step_string?: string;
-    next_step_string?: string;
+  async edit_plan(options: {
+    immediate_action: string;
     notebook_path?: string | null;
-  }): boolean {
+  }): Promise<string | { error: true; errorText: string }> {
     const current = this.notebookTools.getCurrentNotebook(
       options.notebook_path
     );
     if (!current) {
       console.error('No notebook found for edit_plan');
-      return false;
+      return '';
     }
 
     const { notebook } = current;
 
-    // Get the first cell (plan cell)
-    let planCell = null;
-
-    for (let i = 0; i < notebook.widgets.length; i++) {
-      const cell = notebook.widgets[i];
-      if (this.isPlanCell(cell)) {
-        planCell = cell;
-      }
+    // Capture initial state for potential restoration
+    const originalActiveCellIndex = notebook.activeCellIndex;
+    // Get current plan content
+    let currentPlan = '';
+    const firstCell = notebook.widgets[0];
+    if (firstCell) {
+      currentPlan = firstCell.model.sharedModel.getSource();
     }
 
-    if (!planCell) {
-      console.error('No first cell found for plan update');
+    // Get notebook summary
+    const notebookSummary = await this.notebookTools.getNotebookSummary(
+      options.notebook_path || null
+    );
+    const summaryString = JSON.stringify(notebookSummary, null, 2);
+
+    let planExisted = true;
+    let originalMetadata: any = null;
+
+    // Ensure plan cell exists before starting generation
+    if (!firstCell || !this.isPlanCell(firstCell)) {
+      planExisted = false;
       notebook.activeCellIndex = 0;
       const restoreScroll = NotebookCellTools.preventScrolling(notebook);
       NotebookActions.insertAbove(notebook);
@@ -626,32 +634,26 @@ export class NotebookCellTools {
       this.setFirstCellAsPlan(options.notebook_path);
     }
 
-    const firstCell = notebook.widgets[0];
+    const planCell = notebook.widgets[0];
 
-    // Update the plan cell content
-    const oldContent = firstCell.model.sharedModel.getSource();
-    const newContent = options.updated_plan_string;
+    // Capture original metadata for restoration if needed
+    if (planExisted) {
+      originalMetadata = JSON.parse(
+        JSON.stringify(planCell.model.sharedModel.getMetadata() || {})
+      );
+    }
 
     // Get tracking ID for consistent reference
-    const metadata: any = firstCell.model.sharedModel.getMetadata() || {};
+    const metadata: any = planCell.model.sharedModel.getMetadata() || {};
     const trackingId = metadata.cell_tracker?.trackingId || 'planning_cell';
 
-    // Log the diff between old and new content
-    // this.notebookTools['diffTools'].logDiff(
-    //   this.notebookTools.normalizeContent,
-    //   oldContent,
-    //   newContent,
-    //   'edit_plan',
-    //   trackingId
-    // );
-
     // Activate the plan cell
-    this.notebookTools.activateCell(firstCell);
+    this.notebookTools.activateCell(planCell);
 
     // Update metadata
     if (metadata.cell_tracker) {
       metadata.cell_tracker.lastModified = new Date().toISOString();
-      metadata.cell_tracker.summary = 'Updated plan';
+      metadata.cell_tracker.summary = 'Updating plan...';
     } else {
       const now = new Date().toISOString();
       metadata.cell_tracker = {
@@ -659,37 +661,201 @@ export class NotebookCellTools {
         createdAt: now,
         lastModified: now,
         origin: 'ai',
-        summary: 'Updated plan'
+        summary: 'Updating plan...'
       };
     }
 
+    // Update custom summary
     if (!metadata.custom) {
       metadata.custom = {};
     }
-    metadata.custom.summary = 'Updated plan';
-    metadata.custom.current_step_string = options.current_step_string;
-    metadata.custom.next_step_string = options.next_step_string;
-    firstCell.model.sharedModel.setMetadata(metadata);
+    metadata.custom.summary = 'Updating plan...';
+    planCell.model.sharedModel.setMetadata(metadata);
 
-    const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+    // Initialize streaming content
+    let streamedContent = '';
 
-    // Set the new plan content
-    firstCell.model.sharedModel.setSource(newContent);
-    void NotebookActions.runCells(notebook, [firstCell]);
+    // Create the user message with context
+    const userMessage = `## Notebook Summary
+${summaryString}
 
-    restoreScroll();
+## Current Plan
+${currentPlan || 'No existing plan'}
 
-    void AppStateService.getPlanStateDisplay().updatePlan(
-      options.current_step_string,
-      options.next_step_string,
-      options.updated_plan_string
+## Immediate Action
+${options.immediate_action}
+
+Generate an updated plan based on this context.`;
+
+    try {
+      // Get the chat service
+      const chatService = AppStateService.getChatService();
+      const modelName = chatService.getModelName();
+
+      // Load the plan generation prompt
+      // eslint-disable-next-line @typescript-eslint/no-var-requires
+      const planPrompt = require('../Config/prompts/plan_generation_prompt.md');
+
+      // Stream the plan generation with onTextChunk callback
+      const generatedPlan = await chatService.sendEphemeralMessage(
+        userMessage,
+        planPrompt,
+        modelName,
+        (textChunk: string) => {
+          // Append the new chunk to our streamed content
+          streamedContent += textChunk;
+
+          // Update the plan cell content in real-time
+          planCell.model.sharedModel.setSource(streamedContent);
+
+          // Handle smart scrolling to show the updated content
+          this.handleSmartScroll(planCell.node);
+        },
+        { maxTokens: 2048 }
+      );
+
+      // Final content update (in case there are any remaining chunks)
+      const finalContent = generatedPlan.trim();
+      planCell.model.sharedModel.setSource(finalContent);
+
+      const actionHistory = AppStateService.getState().actionHistory;
+      actionHistory?.addActionWithCheckpoint(
+        ActionType.EDIT_PLAN,
+        {
+          planExisted,
+          oldPlan: currentPlan,
+          newContent: finalContent,
+          oldCurrentStep: metadata.custom.current_step_string || '',
+          oldNextStep: metadata.custom.next_step_string || ''
+        },
+        'Updated plan'
+      );
+
+      // Update metadata with final summary
+      if (metadata.cell_tracker) {
+        metadata.cell_tracker.lastModified = new Date().toISOString();
+        metadata.cell_tracker.summary = 'Plan updated';
+      }
+      metadata.custom.summary = 'Plan updated';
+      planCell.model.sharedModel.setMetadata(metadata);
+
+      // Run the plan cell to display it
+      const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+      void NotebookActions.runCells(notebook, [planCell]);
+      restoreScroll();
+
+      // Extract current and next steps from the generated plan
+      const { currentStep, nextStep } = this.extractStepsFromPlan(
+        finalContent,
+        options.immediate_action
+      );
+
+      // Update metadata with extracted steps
+      metadata.custom.current_step_string = currentStep;
+      metadata.custom.next_step_string = nextStep;
+      planCell.model.sharedModel.setMetadata(metadata);
+
+      void AppStateService.getPlanStateDisplay().updatePlan(
+        currentStep || '',
+        nextStep,
+        finalContent
+      );
+
+      console.log('Plan cell updated successfully with streaming');
+      return finalContent;
+    } catch (error: unknown) {
+      console.error('[NotebookCellTools] Failed to generate plan:', error);
+
+      try {
+        // Restore initial state
+        if (!planExisted) {
+          // Plan cell was created during this operation, remove it
+          const restoreScroll = NotebookCellTools.preventScrolling(notebook);
+          notebook.activeCellIndex = 0; // Ensure the plan cell is selected
+          NotebookActions.deleteCells(notebook);
+          notebook.activeCellIndex = originalActiveCellIndex;
+          restoreScroll();
+        } else {
+          // Plan cell existed, restore its original content and metadata
+          const planCell = notebook.widgets[0];
+          if (planCell) {
+            // Restore original content
+            planCell.model.sharedModel.setSource(currentPlan);
+
+            // Restore original metadata
+            if (originalMetadata) {
+              planCell.model.sharedModel.setMetadata(originalMetadata);
+            }
+          }
+
+          // Restore original active cell index
+          notebook.activeCellIndex = originalActiveCellIndex;
+        }
+      } catch (restoreError) {
+        console.error(
+          '[NotebookCellTools] Failed to restore initial state after plan generation error:',
+          restoreError
+        );
+        // Don't throw the restore error, continue with the original error
+      }
+
+      return {
+        error: true,
+        errorText: error instanceof Error ? error.message : String(error)
+      };
+    }
+  }
+
+  /**
+   * Extract current and next steps from a plan based on immediate action
+   */
+  private extractStepsFromPlan(
+    plan: string,
+    immediateAction: string
+  ): {
+    currentStep: string;
+    nextStep: string;
+  } {
+    const lines = plan.split('\n');
+    const tasks: { text: string; completed: boolean }[] = [];
+
+    // Extract all tasks from the plan (skip title line)
+    for (const line of lines) {
+      const taskMatch = line.match(/^-\s*\[\s*([x ])\s*\]\s*(.+)$/);
+      if (taskMatch) {
+        const completed = taskMatch[1] === 'x';
+        const text = taskMatch[2].trim();
+        tasks.push({ text, completed });
+      }
+    }
+
+    // Find current step - either the one being worked on or first incomplete
+    let currentStep = '';
+    let nextStep = '';
+
+    // Look for a task related to the immediate action
+    const actionRelatedTask = tasks.find(
+      task =>
+        !task.completed &&
+        (task.text.toLowerCase().includes(immediateAction.toLowerCase()) ||
+          immediateAction.toLowerCase().includes(task.text.toLowerCase()))
     );
 
-    // Handle smart scrolling after plan update
-    // this.handleSmartScroll(notebook);
+    if (actionRelatedTask) {
+      currentStep = actionRelatedTask.text;
+      // Find the next incomplete task after this one
+      const currentIndex = tasks.indexOf(actionRelatedTask);
+      const nextTask = tasks
+        .slice(currentIndex + 1)
+        .find(task => !task.completed);
+      nextStep = nextTask ? nextTask.text : '';
+    } else {
+      const incompleteTasks = tasks.filter(task => !task.completed);
+      currentStep = incompleteTasks[0]?.text || '';
+      nextStep = incompleteTasks.length > 1 ? incompleteTasks[1]?.text : '';
+    }
 
-    console.log('Plan cell updated successfully');
-    return true;
+    return { currentStep, nextStep };
   }
 
   /**
