@@ -419,13 +419,11 @@ export class NotebookTools {
     return this.cellTools.stream_edit_plan(options);
   }
 
-  edit_plan(options: {
-    updated_plan_string: string;
-    current_step_string?: string;
-    next_step_string?: string;
+  async edit_plan(options: {
+    immediate_action: string;
     notebook_path?: string | null;
-  }): boolean {
-    return this.cellTools.edit_plan(options);
+  }): Promise<string | { error: true; errorText: string }> {
+    return await this.cellTools.edit_plan(options);
   }
 
   get_cells_info(): {
@@ -499,7 +497,21 @@ export class NotebookTools {
   }
 
   /**
-   * Wait for user reply - shows the waiting reply box
+   * Displays a UI component to inform the user that the agent has completed its turn and is now waiting for a user reply.
+   *
+   * This tool should be called as the final action in an assistant's turn after providing a complete response or asking a question.
+   * It signals that the agent is done processing and is awaiting the user's next instruction or answer.
+   *
+   * **How to use `wait_user_reply`:**
+   * 1. **First, send a message** containing your question or the information you want the user to review.
+   * 2. **Immediately after**, call the `wait_user_reply` tool.
+   * 3. **Generate 1-3 follow up responses** that are relevant to the question or action you are waiting for:
+   *    - These should be concise and directly related to the user's potential responses.
+   *    - They should not be speculative or unrelated to the current task.
+   *    - Create exact responses and examples, not vague responses like "Modify the strategy" which can be interpreted in many ways.
+   *    - When asking the user to proceed or continue, only provide one option to continue unless it is extremely relevant to modify the task.
+   *    - For extremely simple tasks such as printing hello world or basic debugging tasks do not include any follow up responses.
+   *
    * @param options Configuration options
    * @param options.notebook_path Path to the notebook file (optional)
    * @param options.recommended_next_prompts List of recommended prompts to show to the user
@@ -510,6 +522,12 @@ export class NotebookTools {
     recommended_next_prompts?: string[];
   }): boolean {
     try {
+      const llmStateDisplay =
+        AppStateService.getChatContainerSafe()?.chatWidget.llmStateDisplay;
+      if (llmStateDisplay) {
+        llmStateDisplay.show('Waiting for your reply...', true);
+      }
+
       // Get the ChatMessages component from the current chat container
       const chatContainer = AppStateService.getState().chatContainer;
       if (chatContainer && chatContainer.chatWidget) {
