@@ -13,20 +13,31 @@ export enum ServiceProvider {
  * Factory for creating chat service instances
  */
 export class ServiceFactory {
+  private static instanceCache: Map<ServiceProvider, IChatService> = new Map();
+
   /**
-   * Create a chat service instance based on provider type
+   * Create or return a cached chat service instance based on provider type
    * @param provider The service provider to use
    * @returns An instance of IChatService
    */
   static createService(provider: ServiceProvider): IChatService {
+    const cached = this.instanceCache.get(provider);
+    if (cached) return cached;
+
+    let instance: IChatService;
     switch (provider) {
       case ServiceProvider.ANTHROPIC:
-        return new AnthropicService();
+        instance = new AnthropicService();
+        break;
       case ServiceProvider.GEMINI:
-        // Placeholder for future Gemini service
-        return new AnthropicService();
+        // Placeholder for future Gemini service; reusing AnthropicService for now
+        instance = new AnthropicService();
+        break;
       default:
         throw new Error(`Unknown service provider: ${provider}`);
     }
+
+    this.instanceCache.set(provider, instance);
+    return instance;
   }
 }
