@@ -40,8 +40,12 @@ import { KernelExecutionListener } from './Chat/ChatContextMenu/KernelExecutionL
 import { IStateDB } from '@jupyterlab/statedb';
 import { v4 as uuidv4 } from 'uuid';
 import { SnippetCreationWidget } from './Components/SnippetCreationWidget';
+import { TabCompletionService } from './Services/TabCompletionService';
+import { CompletionManager } from './Services/CompletionManager';
 import { DiffNavigationWidget } from './Components/DiffNavigationWidget';
+import { IDocumentManager } from '@jupyterlab/docmanager';
 import { timeout } from './utils';
+import { LLMStateDisplay } from './Components/LLMStateDisplay';
 
 const THEME_FLAG_KEY = 'darkThemeApplied';
 
@@ -56,7 +60,13 @@ export const plugin: JupyterFrontEndPlugin<void> = {
   id: 'sage-agent-internal:plugin',
   description: 'Sage AI - Your AI Data Partner',
   autoStart: true,
-  requires: [INotebookTracker, ICommandPalette, IThemeManager, IStateDB],
+  requires: [
+    INotebookTracker,
+    ICommandPalette,
+    IThemeManager,
+    IStateDB,
+    IDocumentManager
+  ],
   optional: [ISettingRegistry, IToolbarWidgetRegistry],
   activate: (
     app: JupyterFrontEnd,
@@ -64,6 +74,7 @@ export const plugin: JupyterFrontEndPlugin<void> = {
     palette: ICommandPalette,
     themeManager: IThemeManager,
     db: IStateDB,
+    documentManager: IDocumentManager,
     settingRegistry: ISettingRegistry | null,
     toolbarRegistry: IToolbarWidgetRegistry | null
   ) => {
@@ -144,73 +155,50 @@ export const plugin: JupyterFrontEndPlugin<void> = {
     };
     checkAndSetTheme();
 
-    // Ensure 'templates' directory exists and create 'rule.default.md' if missing
-    const ensureTemplatesDirAndFile = async () => {
+    // Ensure 'data' directory exists and create 'example.csv' if missing
+    const ensureDataDirAndFile = async () => {
       try {
-        // Check if 'templates' directory exists
+        // Check if 'data' directory exists
         let dirExists = false;
         try {
-          const dir = await contentManager.get('templates');
+          const dir = await contentManager.get('data');
           dirExists = dir.type === 'directory';
         } catch (e) {
           dirExists = false;
         }
         if (!dirExists) {
-          // Create untitled directory, then rename to 'templates'
+          // Create untitled directory, then rename to 'data'
           const untitledDir = await contentManager.newUntitled({
             type: 'directory',
             path: ''
           });
-          await contentManager.rename(untitledDir.path, 'templates');
-          console.log("Created 'templates' directory.");
+          await contentManager.rename(untitledDir.path, 'data');
+          console.log("Created 'data' directory.");
         }
-        // Check if 'rule.default.md' exists
+        // Check if 'example.csv' exists
         let fileExists = false;
         try {
-          await contentManager.get('templates/rule.example.md');
+          await contentManager.get('data/example.csv');
           fileExists = true;
         } catch (e) {
           fileExists = false;
         }
         if (!fileExists) {
-          await contentManager.save('templates/rule.example.md', {
+          await contentManager.save('data/example.csv', {
             type: 'file',
             format: 'text',
             content:
-              '# EXAMPLE TEMPLATE FILE\n' +
-              '\n' +
-              '# Description: \n' +
-              'When called, look into the requested function or codeblock and see if you find any parallelizale code. You can use the following embarringly parallel code template to speed up those function computations\n' +
-              '\n' +
-              '# Code:\n' +
-              '```python\n' +
-              'from joblib import Parallel, delayed\n' +
-              '\n' +
-              'def run_in_batches(fn_name):\n' +
-              '    tickers = get_sp500_tickers()\n' +
-              '    \n' +
-              '    # Process in smaller batches to control memory usage\n' +
-              '    results = Parallel(\n' +
-              '        n_jobs=-1, \n' +
-              '        batch_size=10,  # Process 10 items per batch\n' +
-              "        backend='multiprocessing'\n" +
-              '    )(delayed(test_ticket)(ticker) for ticker in tickers)\n' +
-              '    \n' +
-              '    return dict(zip(tickers, results))\n' +
-              '```'
+              'Date,Open,High,Low,Close,Volume,Symbol\n' +
+              '2024-01-02,473.25,478.90,471.15,477.71,82145600,SPY\n' +
+              '2024-01-03,477.80,481.35,475.20,480.92,73892400,SPY'
           });
-          console.log(
-            "Created 'templates/rule.default.md' with placeholder content."
-          );
+          console.log("Created 'data/example.csv' with S&P 500 sample data.");
         }
       } catch (err) {
-        console.error(
-          'Error ensuring templates directory and rule.default.md:',
-          err
-        );
+        console.error('Error ensuring data directory and example.csv:', err);
       }
     };
-    ensureTemplatesDirAndFile();
+    ensureDataDirAndFile();
 
     // Load settings if available
     if (settingRegistry) {
@@ -242,6 +230,7 @@ export const plugin: JupyterFrontEndPlugin<void> = {
     const toolService = new ToolService();
 
     const planStateDisplay = new PlanStateDisplay();
+    const llmStateDisplay = new LLMStateDisplay();
     const waitingUserReplyBoxManager = new WaitingUserReplyBoxManager();
 
     // Set the did y tracker in the tool service
@@ -278,6 +267,7 @@ export const plugin: JupyterFrontEndPlugin<void> = {
     // Initialize managers in AppState
     AppStateService.initializeManagers(
       planStateDisplay,
+      llmStateDisplay,
       waitingUserReplyBoxManager
     );
 
@@ -293,6 +283,19 @@ export const plugin: JupyterFrontEndPlugin<void> = {
       )
     );
 
+    // Initialize tab completion service (async, non-blocking)
+    const tabCompletionService = TabCompletionService.getInstance();
+    tabCompletionService.initialize().catch(error => {
+      console.warn(
+        '[Plugin] Tab completion service initialization failed:',
+        error
+      );
+    });
+
+    // Initialize completion manager
+    const completionManager = CompletionManager.getInstance();
+    completionManager.initialize(notebooks);
+
     // Initialize database metadata cache (async, non-blocking)
     const databaseCache = DatabaseMetadataCache.getInstance();
     databaseCache.initializeOnStartup().catch(error => {
@@ -460,7 +463,8 @@ export const plugin: JupyterFrontEndPlugin<void> = {
         // Pass the shared tool service, diff manager, and notebook context manager to the container
         const newContainer = new NotebookChatContainer(
           toolService,
-          notebookContextManager
+          notebookContextManager,
+          actionHistory
         );
         await tracker.add(newContainer);
 
@@ -708,20 +712,12 @@ export const plugin: JupyterFrontEndPlugin<void> = {
               ?.next_step_string || '';
           const source = planCell.model.sharedModel.getSource() || '';
 
-          console.log('Updating step floating box', currentStep, nextStep);
-
           void AppStateService.getPlanStateDisplay().updatePlan(
-            currentStep,
+            currentStep || 'Plan active',
             nextStep,
             source,
             false
           );
-        } else if (!planCell) {
-          void AppStateService.getPlanStateDisplay().updatePlan(
-            undefined,
-            undefined,
-            undefined
-          );
         }
 
         notebook?.model?.cells.changed.connect(async () => {
@@ -853,7 +849,7 @@ export const plugin: JupyterFrontEndPlugin<void> = {
 
     // Register all commands
     registerCommands(app, palette);
-    registerEvalCommands(app, palette);
+    registerEvalCommands(app, palette, documentManager);
 
     // Set up notebook tracking to update button state
     notebooks.activeCellChanged.connect((_, cell) => {
@@ -971,7 +967,7 @@ export const plugin: JupyterFrontEndPlugin<void> = {
     const snippetCommandId = 'sage-agent-internal:open-snippet-creation';
 
     commands.addCommand(snippetCommandId, {
-      label: 'Open Snippet Creation',
+      label: 'Open Rule Creation',
       execute: () => {
         if (
           globalSnippetCreationWidget &&
